#define BLYNK_FIRMWARE_VERSION        "1.2.0"
#define BLYNK_PRINT Serial

#define BLYNK_TEMPLATE_ID "TMPLDmd67UA0"
#define BLYNK_DEVICE_NAME "Basils Happiness"
#define BLYNK_AUTH_TOKEN " " // Вказати ваш токен з Blynk

#include <BlynkSimpleEsp32.h>
#include "OTA.h"
#include <WiFi.h>                                        // Библиотека для Wi-Fi
#include "variables.h"

#include <OneWire.h>
#include <DallasTemperature.h>
#define ONE_WIRE_BUS 2
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

#include <SPI.h>
#include <U8g2lib.h>

#include "bsec.h"
Bsec iaqSensor;

U8G2_ST7920_128X64_F_SW_SPI u8g2(U8G2_R2, /* clock=*/ 18, /* data=*/ 23, /* CS=*/ 5, /* reset=*/ 25); // ESP32
// RS - 5
// RW - 23
// E - 18
// RST - 25
// BKL 33

void setup()
{
  Serial.begin(115200);
  Blynk.config(BLYNK_AUTH_TOKEN);

  sensors.begin();
  u8g2.begin();

  Wire.begin(21, 22);

  iaqSensor.begin(BME680_I2C_ADDR_SECONDARY, Wire);

  bsec_virtual_sensor_t sensorList[10] = {
    BSEC_OUTPUT_RAW_TEMPERATURE,
    BSEC_OUTPUT_RAW_PRESSURE,
    BSEC_OUTPUT_RAW_HUMIDITY,
    BSEC_OUTPUT_RAW_GAS,
    BSEC_OUTPUT_IAQ,
    BSEC_OUTPUT_STATIC_IAQ,
    BSEC_OUTPUT_CO2_EQUIVALENT,
    BSEC_OUTPUT_BREATH_VOC_EQUIVALENT,
    BSEC_OUTPUT_SENSOR_HEAT_COMPENSATED_TEMPERATURE,
    BSEC_OUTPUT_SENSOR_HEAT_COMPENSATED_HUMIDITY,
  };

  iaqSensor.updateSubscription(sensorList, 10, BSEC_SAMPLE_RATE_LP);


  pinMode(32, INPUT);
  pinMode(35, INPUT);

  pinMode(RADIATOR_FAN, OUTPUT); // Radiator Fan transistor
  digitalWrite(RADIATOR_FAN, LOW);

  pinMode(DISPLAY_BACKLIGHT, OUTPUT);
  digitalWrite(DISPLAY_BACKLIGHT, HIGH);

  pinMode(RELAY1, OUTPUT);
  digitalWrite(RELAY1, HIGH);
  pinMode(RELAY2, OUTPUT);
  digitalWrite(RELAY2, HIGH);

  pinMode(BUTTON_1, INPUT);
  pinMode(BUTTON_2, INPUT);


  u8g2_prepare();
  Display_Processing(0);

  wifi_connection_status = WiFi_Connect(10000); // connection_timeout = 10000 (10s)
}

void loop()
{
  wifi_health_check();
  Blynk_Processing();


  if (iaqSensor.run()) { // If new data is available

    ambient_temp = iaqSensor.temperature;
    ambient_humi = iaqSensor.humidity;
    ambient_co2 = iaqSensor.co2Equivalent;
    ambient_tvoc = iaqSensor.breathVocEquivalent * 1000;
    atm_pressure = (iaqSensor.pressure) * 0.0075;

    /*
      output += ", " + String(iaqSensor.gasResistance);
      output += ", " + String(iaqSensor.iaq);
      output += ", " + String(iaqSensor.iaqAccuracy);
      output += ", " + String(iaqSensor.staticIaq);
    */
  }

  if (millis() - sensor_read_timer > 5000)
  {
    double moisture_raw_sensor1 = analogRead(32);
    double moisture_raw_sensor2 = analogRead(35);

    moisture_percentage_sensor1 = map(moisture_raw_sensor1, 3500, 1300, 0, 100);
    moisture_percentage_sensor2 = map(moisture_raw_sensor2, 4095, 2900, 0, 100);

    if (moisture_percentage_sensor1 < 0)
      moisture_percentage_sensor1 = 0;
    if (moisture_percentage_sensor2 < 0)
      moisture_percentage_sensor2 = 0;

    wifi_strength = 2 * (WiFi.RSSI() + 100);
    if (wifi_strength > 100)
      wifi_strength = 100;
    if (wifi_strength < 0)
      wifi_strength = 0;

    sensors.requestTemperatures(); // Send the command to get temperatures
    radiator_temp = sensors.getTempCByIndex(0);

    Display_Processing(display_current_page);

    sensor_read_timer = millis();
  }

  if (millis() - blynk_send_timer > 20000 && Blynk.connected())
  {
    Blynk.virtualWrite(V1, radiator_temp);
    Blynk.virtualWrite(V2, ambient_temp);
    Blynk.virtualWrite(V3, ambient_humi);
    Blynk.virtualWrite(V4, moisture_percentage_sensor1);
    Blynk.virtualWrite(V9, moisture_percentage_sensor2);
    Blynk.virtualWrite(V6, ambient_co2);
    Blynk.virtualWrite(V7, ambient_tvoc);
    Blynk.virtualWrite(V8, atm_pressure);
    Blynk.virtualWrite(V10, analogRead(35));

    blynk_send_timer = millis();
  }

  if (millis() - display_backlight_timer > 10000)
    digitalWrite(DISPLAY_BACKLIGHT, LOW);
  else
    digitalWrite(DISPLAY_BACKLIGHT, HIGH);

  if (digitalRead(BUTTON_1) && !button_1_pressed)
  {
    if (display_current_page < 4)
    {
      display_current_page++;
      Display_Processing(display_current_page);
    }
    button_1_pressed = true;
    display_backlight_timer = millis();
  }
  if (!digitalRead(BUTTON_1) && button_1_pressed)
    button_1_pressed = false;

  if (digitalRead(BUTTON_2) && !button_2_pressed)
  {
    if (display_current_page > 1)
    {
      display_current_page--;
      Display_Processing(display_current_page);
    }
    button_2_pressed = true;
    display_backlight_timer = millis();
  }
  if (!digitalRead(BUTTON_2) && button_2_pressed)
    button_2_pressed = false;


  LED_ThermalProtection(50);        // critical_temp = 50
  FAN_Control(30, 30);              // off_temp = 30, on_temp = 35

  LED_Processing();
  FAN_Processing();
  Waterpump_Processing();
}
